const {
    OPCUAClient,
    AttributeIds,
    TimestampsToReturn,
    StatusCodes,
    DataType,
    ClientSubscription
} = require("node-opcua");

// Server URL
const endpointUrl = "opc.tcp://localhost:4870"; 
//"opc.tcp://opcuademo.sterfive.com:26543";
// Node id to read/write
const nodeId = "ns=4;s=SCADA_pec_senzor_temperatura"; 
//"ns=8;s=Scalar_Simulation_Double";
//const nodeId = "ns=4;s=Kljuc";

async function main() {

    try {

        const client = OPCUAClient.create({
            endpointMustExist: false,
            connectionStrategy: {
                maxRetry: 2,
                initialDelay: 2000,
                maxDelay: 10 * 1000
            }
        });
        client.on("backoff", () => console.log("retrying connection"));


        await client.connect(endpointUrl);

        const session = await client.createSession();

        const browseResult = await session.browse("RootFolder");

        console.log("Root children:\n ", browseResult.references.map((r) => r.browseName.toString()).join("\n  "));

        const dataValue = await session.read({ nodeId, attributeId: AttributeIds.Value });
        console.log(`node ${nodeId} value = ${dataValue.value.value.toString()}`);

        
        // const subscription = new ClientSubscription(session,{
        //     requestedPublishingInterval: 10,
        //     requestedMaxKeepAliveCount:  2000,
        //     requestedLifetimeCount:      6000,
        //     maxNotificationsPerPublish:  1000,
        //     publishingEnabled: true,
        //     priority: 10
        // });
        
        
        // Install a subscription and monitored item
        const subscription = await session.createSubscription2({
            requestedPublishingInterval: 1000,
            requestedLifetimeCount: 100, // 1000ms *100 every 2 minutes or so
            requestedMaxKeepAliveCount: 10,// every 10 seconds
            maxNotificationsPerPublish: 10,
            publishingEnabled: true,
            priority: 10
        });

        
        subscription
            .on("started", () => console.log("subscription started - subscriptionId=", subscription.subscriptionId))
            .on("keepalive", () => console.log("keepalive"))
            .on("terminated", () => console.log("subscription terminated"));


        const monitoredItem = await subscription.monitor({
            nodeId: nodeId,
            attributeId: AttributeIds.Value
        },
            {
                samplingInterval: 1000,
                discardOldest: true,
                queueSize: 10
            }, TimestampsToReturn.Both);


        monitoredItem.on("changed", (dataValue) =>
            console.log(` value = ${dataValue.value.value.toString()}`));

        // Wait for 5 seconds
        await new Promise((resolve) => setTimeout(resolve, 5000));

        await subscription.terminate();

        const statusCode = await session.write({
            nodeId: nodeId,
            attributeId: AttributeIds.Value,
            value: {
                statusCode: StatusCodes.Good,
                sourceTimestamp: new Date(),
                value: {
                    dataType: DataType.Double,
                    value: 25.0
                }
            }
        });
        console.log("statusCode = ", statusCode.toString());

        console.log(" closing session");
        await session.close();

        await client.disconnect();
    }
    catch (err) {
        console.log("Error !!!", err);
        process.exit();
    }
}

main();
